@@ -14,16 +14,18 @@ def build_genome_blastdb(genome_id):
 @transaction.atomic()
 def annotate_integration_on_genome(genome, new_genome,
                                    regions_before, regions_after,
-                                   cassette_name, op):
-    annotate_integration(genome, new_genome, regions_before, regions_after, cassette_name, op)
+                                   cassette_name, op, cassette, annotations):
+    annotate_integration(genome, new_genome, regions_before, regions_after,
+                         cassette_name, op, cassette, annotations)
 
 
 @shared_task
 def annotate_integration_task(genome_id, new_genome_id,
                               regions_before, regions_after,
-                              cassette_name, op_id):
+                              cassette_name, op_id, cassette, annotations):
     genome = Genome.objects.get(pk=genome_id)
     new_genome = Genome.objects.get(pk=new_genome_id)
     op = Operation.objects.get(pk=op_id)
     annotate_integration_on_genome(genome, new_genome,
-                                   regions_before, regions_after, cassette_name, op)
+                                   regions_before, regions_after,
+                                   cassette_name, op, cassette, annotations)
